/* global Bdf,$$ */
/**
 * Objet global définissant l'espace de nom Ficheform
 * 
 * @namespace Ficheframe,Bdf
 */
var Ficheframe = {};

Ficheframe.backgroundColor = "#f4f4f4";
Ficheframe.startingLeft = 5;

Ficheframe.ARGS = {
    goto: ""
};

Ficheframe.checkFieldeditDim = function (fieldeditId) {
    Ficheframe.startingLeft = parseInt($$(fieldeditId).css("left"));
};

Ficheframe.testToolbar = function (iframe, toolbarId, fieldeditId, editTarget, tocTarget, callback) {
    var contentWindow = iframe.contentWindow;
    _showToolbar(false);
    var buttonId = "ficheframe_button_ficheedit";
    var pathName;
    try {
        pathName = contentWindow.location.pathname;
    } catch(error) {
        return;
    }
    if (!pathName.startsWith(_getFichothequeRoot())) {
        return;
    }
    var ficheTest = pathName.match(/\/fiches\/([a-z][a-z0-9]*)-([0-9]+)(-[_a-z0-9]+)?\.html$/);
    if (!ficheTest) {
        return;
    }
    var corpusName = ficheTest[1];
    var ficheId = ficheTest[2];
    var positions = null;
    Bdf.api.loadFiche({
        corpus: corpusName,
        id: ficheId,
        callback: function (data) {
            if (data.fiche) {
                let editable = data.fiche.editable;
                if (editable) {
                    _checkFields();
                } else {
                    $$(fieldeditId).html("");
                }
                let toc = _checkToc(contentWindow.document);
                let html = Bdf.render("ficheframe:toolbar", {
                    path: pathName,
                    corpusName: corpusName,
                    ficheId: ficheId,
                    editable: editable,
                    toc: toc,
                    editTarget: editTarget,
                    tocTarget: tocTarget
                });
                $$(toolbarId).html(html);
                if (html.length > 0) {
                    _showToolbar(true);
                    $(contentWindow).on('pagehide', function(e) {
                        _showToolbar(false);
                        _hideFieldEdit();
                    });
                    let editLink = $$.one(buttonId);
                    let originalHref = editLink.href;
                    if (toc.length > 0) {
                        $(contentWindow).on("scroll", function(event) {
                            let threshold = contentWindow.innerHeight * 2 / 3;
                            let scrollTop = $(contentWindow.document).scrollTop();
                            let currentTocItem = false;
                            for (let i = toc.length-1; i >= 0; i--) {
                                let tocItem = toc[i];
                                if (scrollTop > $(tocItem.element).offset().top - threshold) {
                                    currentTocItem = tocItem;
                                    if (i > 0) {
                                        let previousTocItem = toc[i - 1];
                                        let $previousEl = $(previousTocItem.element);
                                        let previousBottom = $previousEl.offset().top + $previousEl.height() + (threshold / 3);
                                        if (previousBottom > scrollTop) {
                                            currentTocItem = previousTocItem;
                                        }
                                    }
                                    break;
                                }
                            }
                            $(".ficheframe-toc-Active").removeClass("ficheframe-toc-Active");
                            if (currentTocItem) {
                                $$("linkto:" + currentTocItem.id).addClass("ficheframe-toc-Active");
                                editLink.href = originalHref + "&goto=" + currentTocItem.goto;
                            } else {
                                editLink.href = originalHref;
                            }
                        });
                    }
                }
                if (callback) {
                    callback();
                }
            }
        }
    });
    
    function _checkFields() {
        let timer;
        $(contentWindow).on("mousemove", function(event) {
            if (timer) {
                clearTimeout(timer);
                timer = false;
            }
            if (event.buttons) {
                _hideFieldEdit();
                return;
            }
            let currentSelection = contentWindow.getSelection();
            if ((currentSelection) && (currentSelection.type === "Range")) {
                _hideFieldEdit();
                return;
            }
            let target = event.target;
            if ((target) && (target.localName === "a")) {
                 _hideFieldEdit();
                return;
            }
            if ((positions) && (positions.isChanged(event.pageY))) {
                _hideFieldEdit();
            }
            if (event.pageX < 30) {
                __testPosition(event);
            } else {
                timer = setTimeout(function () {
                    __testPosition(event);
                }, 150);            
            }
        });
        $(contentWindow).on("scroll resize", function() {
            if (positions) {
                positions.clear();
                _hideFieldEdit();
                positions = null;
            }
        });
        
        function __testPosition(event) {
            let pageY = event.pageY;
            if (!positions) {
                positions = Ficheframe.Positions.build(contentWindow.document);
            }
            if (positions.isChanged(pageY)) {
                let newPosition = positions.updatePosition(pageY);
                if ((newPosition) && (newPosition.goto)) {
                    let top = newPosition.startY - $(contentWindow.document).scrollTop() + $(iframe).position().top;
                    if (top > 3) {
                        positions.activateCurrent(true);
                        $$(fieldeditId).html(Bdf.render("ficheframe:fieldedit", {
                            corpusName: corpusName,
                            ficheId: ficheId,
                            editTarget: editTarget,
                            goto: newPosition.goto
                        })).css("top", top).css("left", ___getLeft(newPosition));
                    }
                }
            }
            _checkFieldEdit();
            
            function ___getLeft(newPosition) {
                if (newPosition.startX > 50) {
                    return Ficheframe.startingLeft + newPosition.startX - 50;
                }
                return Ficheframe.startingLeft;
            }
        }
    }
    
    function _showToolbar(display) {
        Ficheframe.show(toolbarId, display);
    }
    
    function _hideFieldEdit() {
        Ficheframe.show(fieldeditId, false);
        if (positions) {
            positions.highlightCurrent(false);
        }
    }
    
    function _checkFieldEdit() {
        let display = positions.isCurrentActive;
        Ficheframe.show(fieldeditId, display);
        positions.highlightCurrent(display);
    }
    
    function _checkToc(document) {
        let array = new Array();
        $$(document, {bdfFormMilestone: true}).each(function (index, element) {
            let milestoneType = element.dataset.bdfFormMilestone;
            let goto;
            let $headings;
            if (milestoneType === "satellite") {
                goto = "satellite_" + element.dataset.bdfSatellite;
                $headings = $(element).children("h1");
            } else if (milestoneType === "comment") {
                goto = element.dataset.bdfComponentName;
                let $satellite = $(element).parents($$.toCssSelector({bdfFormMilestone: "satellite"}));
                if ($satellite.length > 0) {
                    goto = "satellite_" + $satellite[0].dataset.bdfSatellite + "/" + goto;
                }
                $headings = $(element).find("h2");
            }
            if ($headings) {
                $headings.each(function (index2, element2) {
                    let id = element2.id;
                    if (!id) {
                        id = Bdf.generateId();
                        element2.id = id;
                    }
                    let $element2 = $(element2);
                    let text = $element2.text();
                    if (!$element2.is(":hidden")) {
                        array.push({
                            id: id,
                            text: text,
                            element: element2,
                            goto: goto
                        });
                    }
                });
            }
        });
        return array;
    }
    
    function _getFichothequeRoot() {
        let currentPathName = window.location.pathname;
        let lastIndex = currentPathName.lastIndexOf("/");
        return currentPathName.substr(0, lastIndex + 1);
    }
};

Ficheframe.show = function (elementId, display) {
    if (display) {
        $$(elementId).removeClass("hidden");
    } else {
        $$(elementId).addClass("hidden");
    }
};

Ficheframe.Position = function (startY, endY, element, goto, startX) {
    this.startY = startY;
    this.endY = endY;
    this.element = element;
    this.goto = goto;
    this.startMargin = startY;
    this.endMargin = endY;
    this.startX = startX;
};

Ficheframe.Position.prototype.contains = function (y) {
    if ((y >= this.startY) && (y <= this.endY)) {
        return true;
    } else {
        return false;
    }
};

Ficheframe.Position.prototype.containsWithMargin = function (y) {
    if ((y >= this.startMargin) && (y <= this.endMargin)) {
        return true;
    } else {
        return false;
    }
};

Ficheframe.Position.prototype.compare = function (y) {
    if (y < this.startY) {
        return -1;
    }
    if (y > this.endY) {
        return 1;
    }
    return 0;
};

Ficheframe.Position.prototype.setBackground = function (color) {
    this.element.style.backgroundColor = color;
};

Ficheframe.Positions = function (positionArray) {
  this.positionArray = positionArray; 
  this.currentIndex = -1;
  this.currentPosition = null;
  this.isCurrentActive = false;
};

Ficheframe.Positions.prototype.isChanged = function (pageY) {
    var currentPosition = this.currentPosition;
    if ((currentPosition) && (currentPosition.containsWithMargin(pageY))) {
        return false;
    }
    return true;
};

Ficheframe.Positions.prototype.activateCurrent = function () {
    this.isCurrentActive = true;
};

Ficheframe.Positions.prototype.clear = function () {
    if (this.currentPosition) {
        this.currentPosition.setBackground(null);
        this.currentPosition = null;
        this.currentIndex = -1;
        this.isCurrentActive = false;
    }
};

Ficheframe.Positions.prototype.updatePosition = function (pageY) {
    this.clear();
    let length = this.positionArray.length;
    for(let i = 0; i < length; i++) {
        let position = this.positionArray[i];
        let comp = position.compare(pageY);
        if (comp === 0) {
            this.currentPosition = position;
            this.currentIndex = i;
            break;
        } else if (comp < 0) {
            break;
        }
    }
    return this.currentPosition;
};

Ficheframe.Positions.prototype.highlightCurrent = function (highlight) {
    if (this.currentPosition) {
        let backgroundColor = null;
        if (highlight) {
            backgroundColor = Ficheframe.backgroundColor;
        }
        this.currentPosition.setBackground(backgroundColor);
    }
};

Ficheframe.Positions.build = function (document) {
    var positionArray = new Array();
    $(document).find($$.toCssSelector({bdfField: true})).each(function (index, element) {
        let fieldType = element.dataset.bdfField;
        let goto = _getGoto(element);
        let done = false;
        if (fieldType === "section") {
            let firstDim;
            $$(element, {bdfPosition: true}).each(function (index, ficheblock) {
                let dim = _getDim(ficheblock);
                if (!firstDim) {
                    firstDim = dim;
                }
                positionArray.push(new Ficheframe.Position(dim[0], dim[1], ficheblock, goto + "@" + ficheblock.dataset.bdfPosition, dim[2]));
            });
            if (firstDim) {
                done = true;
                let dim = _getDim(element);
                positionArray.push(new Ficheframe.Position(dim[0], firstDim[0] - 1, element, goto, dim[2]));
            }
        }
        if (!done) {
            let dim = _getDim(element);
            positionArray.push(new Ficheframe.Position(dim[0], dim[1], element, goto, dim[2]));
        }
    });
    positionArray.sort(function (p1, p2) {
        if (p1.startY < p2.startY) {
            return -1;
        }
        if (p1.startY > p2.startY) {
            return 1;
        }
        if (p1.endY < p2.endY) {
            return -1;
        }
        if (p1.endY > p2.endY) {
            return 1;
        }
        return 0;
    });
    let length = positionArray.length;
    let marginMax = 12;
    for(let i = 0; i < length; i++) {
        let currentPosition = positionArray[i];
        if (i === 0) {
            currentPosition.startMargin = Math.max(0, currentPosition.startY -  marginMax);
        } else {
            let previous = positionArray[i -1];
            let margin = Math.min(marginMax, (currentPosition.startY - previous.endY) / 2);
            if (margin < 0) {
                margin = 0;
            }
            currentPosition.startMargin = currentPosition.startY -  margin;
        }
        if (i === (length - 1)) {
            currentPosition.endMargin = Math.min($(document).height(), currentPosition.endY +  marginMax);
        } else {
            let next = positionArray[i + 1];
            let margin = Math.min(marginMax, (next.startY - currentPosition.endY) / 2);
            if (margin < 0) {
                margin = 0;
            }
            currentPosition.endMargin = Math.max(Math.min(positionArray[i + 1].startY - 1, currentPosition.endY +  margin), currentPosition.endY);
        }
    }
    return new Ficheframe.Positions(positionArray);
    
    function _getDim(element) {
        let $element = $(element);
        let offset = $element.offset();
        let startY = offset.top;
        let array = new Array();
        array.push(startY);
        array.push(startY + $element.height());
        array.push(offset.left);
        return array;
    }
    
    function _getGoto(element) {
        let goto = element.dataset.bdfComponentName;
        if (!goto) {
            return null;
        }
        if (goto === "liage") {
            if (element.dataset.bdfField !== "corpus") {
                return null;
            }
        } else if (goto.startsWith("_master_")) {
            if (!goto.startsWith("_master_" + element.dataset.bdfField)) {
                return null;
            }
        } else if (!goto.startsWith(element.dataset.bdfField)) {
            return null;
        }
        let $satellite = $(element).parents($$.toCssSelector({bdfFormMilestone: "satellite"}));
        if ($satellite.length > 0) {
            goto = "satellite_" + $satellite[0].dataset.bdfSatellite + "/" + goto;
        }
        return goto;
    }
};
